// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_FEED_INTERNALS_FEED_INTERNALS_MOJOM_SHARED_H_
#define CHROME_BROWSER_UI_WEBUI_FEED_INTERNALS_FEED_INTERNALS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/browser/ui/webui/feed_internals/feed_internals.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace feed_internals {
namespace mojom {
class PropertiesDataView;

class UserClassifierDataView;

class LastFetchPropertiesDataView;

class SuggestionDataView;

class TimeDataView;



}  // namespace mojom
}  // namespace feed_internals

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::feed_internals::mojom::PropertiesDataView> {
  using Data = ::feed_internals::mojom::internal::Properties_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::feed_internals::mojom::UserClassifierDataView> {
  using Data = ::feed_internals::mojom::internal::UserClassifier_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::feed_internals::mojom::LastFetchPropertiesDataView> {
  using Data = ::feed_internals::mojom::internal::LastFetchProperties_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::feed_internals::mojom::SuggestionDataView> {
  using Data = ::feed_internals::mojom::internal::Suggestion_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::feed_internals::mojom::TimeDataView> {
  using Data = ::feed_internals::mojom::internal::Time_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace feed_internals {
namespace mojom {
// Interface base classes. They are used for type safety check.
class PageHandlerInterfaceBase {};

using PageHandlerPtrDataView =
    mojo::InterfacePtrDataView<PageHandlerInterfaceBase>;
using PageHandlerRequestDataView =
    mojo::InterfaceRequestDataView<PageHandlerInterfaceBase>;
using PageHandlerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageHandlerInterfaceBase>;
using PageHandlerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageHandlerInterfaceBase>;
class PropertiesDataView {
 public:
  PropertiesDataView() {}

  PropertiesDataView(
      internal::Properties_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool is_feed_enabled() const {
    return data_->is_feed_enabled;
  }
 private:
  internal::Properties_Data* data_ = nullptr;
};

class UserClassifierDataView {
 public:
  UserClassifierDataView() {}

  UserClassifierDataView(
      internal::UserClassifier_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUserClassDescriptionDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUserClassDescription(UserType* output) {
    auto* pointer = data_->user_class_description.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  float avg_hours_between_views() const {
    return data_->avg_hours_between_views;
  }
  float avg_hours_between_uses() const {
    return data_->avg_hours_between_uses;
  }
 private:
  internal::UserClassifier_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class LastFetchPropertiesDataView {
 public:
  LastFetchPropertiesDataView() {}

  LastFetchPropertiesDataView(
      internal::LastFetchProperties_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t last_fetch_status() const {
    return data_->last_fetch_status;
  }
  inline void GetLastFetchTimeDataView(
      TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLastFetchTime(UserType* output) {
    auto* pointer = data_->last_fetch_time.Get();
    return mojo::internal::Deserialize<::feed_internals::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetRefreshSuppressTimeDataView(
      TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRefreshSuppressTime(UserType* output) {
    auto* pointer = data_->refresh_suppress_time.Get();
    return mojo::internal::Deserialize<::feed_internals::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::LastFetchProperties_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class SuggestionDataView {
 public:
  SuggestionDataView() {}

  SuggestionDataView(
      internal::Suggestion_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTitleDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTitle(UserType* output) {
    auto* pointer = data_->title.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetPublisherNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPublisherName(UserType* output) {
    auto* pointer = data_->publisher_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetImageUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadImageUrl(UserType* output) {
    auto* pointer = data_->image_url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetFaviconUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFaviconUrl(UserType* output) {
    auto* pointer = data_->favicon_url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::Suggestion_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TimeDataView {
 public:
  TimeDataView() {}

  TimeDataView(
      internal::Time_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  double ms_since_epoch() const {
    return data_->ms_since_epoch;
  }
 private:
  internal::Time_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace feed_internals

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::feed_internals::mojom::PropertiesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::feed_internals::mojom::PropertiesDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::feed_internals::mojom::internal::Properties_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->is_feed_enabled = CallWithContext(Traits::is_feed_enabled, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::feed_internals::mojom::internal::Properties_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::feed_internals::mojom::PropertiesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::feed_internals::mojom::UserClassifierDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::feed_internals::mojom::UserClassifierDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::feed_internals::mojom::internal::UserClassifier_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::user_class_description, input, custom_context)) in_user_class_description = CallWithContext(Traits::user_class_description, input, custom_context);
    typename decltype((*output)->user_class_description)::BaseType::BufferWriter
        user_class_description_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_user_class_description, buffer, &user_class_description_writer, context);
    (*output)->user_class_description.Set(
        user_class_description_writer.is_null() ? nullptr : user_class_description_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->user_class_description.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null user_class_description in UserClassifier struct");
    (*output)->avg_hours_between_views = CallWithContext(Traits::avg_hours_between_views, input, custom_context);
    (*output)->avg_hours_between_uses = CallWithContext(Traits::avg_hours_between_uses, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::feed_internals::mojom::internal::UserClassifier_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::feed_internals::mojom::UserClassifierDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::feed_internals::mojom::LastFetchPropertiesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::feed_internals::mojom::LastFetchPropertiesDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::feed_internals::mojom::internal::LastFetchProperties_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->last_fetch_status = CallWithContext(Traits::last_fetch_status, input, custom_context);
    decltype(CallWithContext(Traits::last_fetch_time, input, custom_context)) in_last_fetch_time = CallWithContext(Traits::last_fetch_time, input, custom_context);
    typename decltype((*output)->last_fetch_time)::BaseType::BufferWriter
        last_fetch_time_writer;
    mojo::internal::Serialize<::feed_internals::mojom::TimeDataView>(
        in_last_fetch_time, buffer, &last_fetch_time_writer, context);
    (*output)->last_fetch_time.Set(
        last_fetch_time_writer.is_null() ? nullptr : last_fetch_time_writer.data());
    decltype(CallWithContext(Traits::refresh_suppress_time, input, custom_context)) in_refresh_suppress_time = CallWithContext(Traits::refresh_suppress_time, input, custom_context);
    typename decltype((*output)->refresh_suppress_time)::BaseType::BufferWriter
        refresh_suppress_time_writer;
    mojo::internal::Serialize<::feed_internals::mojom::TimeDataView>(
        in_refresh_suppress_time, buffer, &refresh_suppress_time_writer, context);
    (*output)->refresh_suppress_time.Set(
        refresh_suppress_time_writer.is_null() ? nullptr : refresh_suppress_time_writer.data());
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::feed_internals::mojom::internal::LastFetchProperties_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::feed_internals::mojom::LastFetchPropertiesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::feed_internals::mojom::SuggestionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::feed_internals::mojom::SuggestionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::feed_internals::mojom::internal::Suggestion_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::title, input, custom_context)) in_title = CallWithContext(Traits::title, input, custom_context);
    typename decltype((*output)->title)::BaseType::BufferWriter
        title_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_title, buffer, &title_writer, context);
    (*output)->title.Set(
        title_writer.is_null() ? nullptr : title_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->title.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null title in Suggestion struct");
    decltype(CallWithContext(Traits::url, input, custom_context)) in_url = CallWithContext(Traits::url, input, custom_context);
    typename decltype((*output)->url)::BaseType::BufferWriter
        url_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_url, buffer, &url_writer, context);
    (*output)->url.Set(
        url_writer.is_null() ? nullptr : url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null url in Suggestion struct");
    decltype(CallWithContext(Traits::publisher_name, input, custom_context)) in_publisher_name = CallWithContext(Traits::publisher_name, input, custom_context);
    typename decltype((*output)->publisher_name)::BaseType::BufferWriter
        publisher_name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_publisher_name, buffer, &publisher_name_writer, context);
    (*output)->publisher_name.Set(
        publisher_name_writer.is_null() ? nullptr : publisher_name_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->publisher_name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null publisher_name in Suggestion struct");
    decltype(CallWithContext(Traits::image_url, input, custom_context)) in_image_url = CallWithContext(Traits::image_url, input, custom_context);
    typename decltype((*output)->image_url)::BaseType::BufferWriter
        image_url_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_image_url, buffer, &image_url_writer, context);
    (*output)->image_url.Set(
        image_url_writer.is_null() ? nullptr : image_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->image_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null image_url in Suggestion struct");
    decltype(CallWithContext(Traits::favicon_url, input, custom_context)) in_favicon_url = CallWithContext(Traits::favicon_url, input, custom_context);
    typename decltype((*output)->favicon_url)::BaseType::BufferWriter
        favicon_url_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_favicon_url, buffer, &favicon_url_writer, context);
    (*output)->favicon_url.Set(
        favicon_url_writer.is_null() ? nullptr : favicon_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->favicon_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null favicon_url in Suggestion struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::feed_internals::mojom::internal::Suggestion_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::feed_internals::mojom::SuggestionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::feed_internals::mojom::TimeDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::feed_internals::mojom::TimeDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::feed_internals::mojom::internal::Time_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->ms_since_epoch = CallWithContext(Traits::ms_since_epoch, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::feed_internals::mojom::internal::Time_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::feed_internals::mojom::TimeDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace feed_internals {
namespace mojom {



inline void UserClassifierDataView::GetUserClassDescriptionDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->user_class_description.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void LastFetchPropertiesDataView::GetLastFetchTimeDataView(
    TimeDataView* output) {
  auto pointer = data_->last_fetch_time.Get();
  *output = TimeDataView(pointer, context_);
}
inline void LastFetchPropertiesDataView::GetRefreshSuppressTimeDataView(
    TimeDataView* output) {
  auto pointer = data_->refresh_suppress_time.Get();
  *output = TimeDataView(pointer, context_);
}


inline void SuggestionDataView::GetTitleDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->title.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void SuggestionDataView::GetUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->url.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void SuggestionDataView::GetPublisherNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->publisher_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void SuggestionDataView::GetImageUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->image_url.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void SuggestionDataView::GetFaviconUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->favicon_url.Get();
  *output = mojo::StringDataView(pointer, context_);
}





}  // namespace mojom
}  // namespace feed_internals

#endif  // CHROME_BROWSER_UI_WEBUI_FEED_INTERNALS_FEED_INTERNALS_MOJOM_SHARED_H_